/****************************************************************************
** Meta object code from reading C++ file 'camerasettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../camerasettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'camerasettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14CameraSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto CameraSettings::qt_create_metaobjectdata<qt_meta_tag_ZN14CameraSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CameraSettings",
        "resolutionChanged",
        "",
        "cameraDeviceIdChanged",
        "cameraPositionChanged",
        "whiteBalanceModeChanged",
        "videoRecordingFpsChanged",
        "videoRecordingQualityChanged",
        "videoResolutionChanged",
        "videoCodecChanged",
        "filenamePatternChanged",
        "outputPathChanged",
        "resolution",
        "QSize",
        "isResolutionImmutable",
        "cameraDeviceId",
        "isCameraDeviceIdImmutable",
        "cameraPosition",
        "isCameraPositionImmutable",
        "whiteBalanceMode",
        "isWhiteBalanceModeImmutable",
        "videoRecordingFps",
        "isVideoRecordingFpsImmutable",
        "defaultVideoRecordingFpsValue",
        "videoRecordingQuality",
        "isVideoRecordingQualityImmutable",
        "defaultVideoRecordingQualityValue",
        "videoResolution",
        "isVideoResolutionImmutable",
        "defaultVideoResolutionValue",
        "videoCodec",
        "isVideoCodecImmutable",
        "defaultVideoCodecValue",
        "filenamePattern",
        "isFilenamePatternImmutable",
        "defaultFilenamePatternValue",
        "outputPath",
        "isOutputPathImmutable",
        "defaultOutputPathValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resolutionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cameraDeviceIdChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cameraPositionChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'whiteBalanceModeChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoRecordingFpsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoRecordingQualityChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoResolutionChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoCodecChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'filenamePatternChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputPathChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'resolution'
        QtMocHelpers::PropertyData<QSize>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'isResolutionImmutable'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cameraDeviceId'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isCameraDeviceIdImmutable'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cameraPosition'
        QtMocHelpers::PropertyData<int>(17, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isCameraPositionImmutable'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'whiteBalanceMode'
        QtMocHelpers::PropertyData<int>(19, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isWhiteBalanceModeImmutable'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'videoRecordingFps'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isVideoRecordingFpsImmutable'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultVideoRecordingFpsValue'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'videoRecordingQuality'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isVideoRecordingQualityImmutable'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultVideoRecordingQualityValue'
        QtMocHelpers::PropertyData<int>(26, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'videoResolution'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isVideoResolutionImmutable'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultVideoResolutionValue'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'videoCodec'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'isVideoCodecImmutable'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultVideoCodecValue'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'filenamePattern'
        QtMocHelpers::PropertyData<int>(33, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'isFilenamePatternImmutable'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultFilenamePatternValue'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'outputPath'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'isOutputPathImmutable'
        QtMocHelpers::PropertyData<bool>(37, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultOutputPathValue'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CameraSettings, qt_meta_tag_ZN14CameraSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CameraSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CameraSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CameraSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14CameraSettingsE_t>.metaTypes,
    nullptr
} };

void CameraSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CameraSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resolutionChanged(); break;
        case 1: _t->cameraDeviceIdChanged(); break;
        case 2: _t->cameraPositionChanged(); break;
        case 3: _t->whiteBalanceModeChanged(); break;
        case 4: _t->videoRecordingFpsChanged(); break;
        case 5: _t->videoRecordingQualityChanged(); break;
        case 6: _t->videoResolutionChanged(); break;
        case 7: _t->videoCodecChanged(); break;
        case 8: _t->filenamePatternChanged(); break;
        case 9: _t->outputPathChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::resolutionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::cameraDeviceIdChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::cameraPositionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::whiteBalanceModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::videoRecordingFpsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::videoRecordingQualityChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::videoResolutionChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::videoCodecChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::filenamePatternChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (CameraSettings::*)()>(_a, &CameraSettings::outputPathChanged, 9))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QSize*>(_v) = _t->resolution(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isResolutionImmutable(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->cameraDeviceId(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isCameraDeviceIdImmutable(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->cameraPosition(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isCameraPositionImmutable(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->whiteBalanceMode(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->isWhiteBalanceModeImmutable(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->videoRecordingFps(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->isVideoRecordingFpsImmutable(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->defaultVideoRecordingFpsValue(); break;
        case 11: *reinterpret_cast<int*>(_v) = _t->videoRecordingQuality(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->isVideoRecordingQualityImmutable(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->defaultVideoRecordingQualityValue(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->videoResolution(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->isVideoResolutionImmutable(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->defaultVideoResolutionValue(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->videoCodec(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isVideoCodecImmutable(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->defaultVideoCodecValue(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->filenamePattern(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->isFilenamePatternImmutable(); break;
        case 22: *reinterpret_cast<int*>(_v) = _t->defaultFilenamePatternValue(); break;
        case 23: *reinterpret_cast<int*>(_v) = _t->outputPath(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->isOutputPathImmutable(); break;
        case 25: *reinterpret_cast<int*>(_v) = _t->defaultOutputPathValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setResolution(*reinterpret_cast<QSize*>(_v)); break;
        case 2: _t->setCameraDeviceId(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setCameraPosition(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setWhiteBalanceMode(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setVideoRecordingFps(*reinterpret_cast<int*>(_v)); break;
        case 11: _t->setVideoRecordingQuality(*reinterpret_cast<int*>(_v)); break;
        case 14: _t->setVideoResolution(*reinterpret_cast<int*>(_v)); break;
        case 17: _t->setVideoCodec(*reinterpret_cast<int*>(_v)); break;
        case 20: _t->setFilenamePattern(*reinterpret_cast<int*>(_v)); break;
        case 23: _t->setOutputPath(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *CameraSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CameraSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CameraSettingsE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int CameraSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void CameraSettings::resolutionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CameraSettings::cameraDeviceIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CameraSettings::cameraPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CameraSettings::whiteBalanceModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void CameraSettings::videoRecordingFpsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void CameraSettings::videoRecordingQualityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void CameraSettings::videoResolutionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void CameraSettings::videoCodecChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void CameraSettings::filenamePatternChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void CameraSettings::outputPathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
