/****************************************************************************
** Meta object code from reading C++ file 'plasmacamera.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasma-camera-26.04.0/src/plasmacamera.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QQueue>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plasmacamera.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12PlasmaCameraE_t {};
} // unnamed namespace

template <> constexpr inline auto PlasmaCamera::qt_create_metaobjectdata<qt_meta_tag_ZN12PlasmaCameraE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlasmaCamera",
        "errorChanged",
        "",
        "errorOccurred",
        "errorString",
        "cameraDeviceListChanged",
        "availableChanged",
        "available",
        "cameraDeviceChanged",
        "cameraDevice",
        "cameraFormatChanged",
        "busyChanged",
        "viewFinderFrame",
        "QImage",
        "frame",
        "stillCaptureFrames",
        "QQueue<QImage>",
        "frames",
        "afWindowChanged",
        "QSize",
        "newAfWindow",
        "settingsChanged",
        "Settings",
        "settings",
        "fpsChanged",
        "fps",
        "setCameraDevice",
        "cameraDeviceId",
        "setCameraFormat",
        "QCameraFormat",
        "cameraFormat",
        "setAfWindow",
        "afWindow",
        "resetAfWindow",
        "setIso",
        "iso",
        "setExposureTime",
        "exposureTime",
        "setExposureValue",
        "exposureValue",
        "resetIso",
        "resetExposureTime",
        "resetExposureValue",
        "setWbTemp",
        "wbTemp",
        "resetAwb",
        "setContrast",
        "contrast",
        "setSaturation",
        "saturation",
        "resetContrast",
        "resetSaturation",
        "setFps",
        "setError",
        "unsetError",
        "handleCameraAdded",
        "std::shared_ptr<libcamera::Camera>",
        "camera",
        "handleCameraRemoved",
        "switchToNextCamera",
        "startCamera",
        "stopCamera",
        "error",
        "cameraDeviceIds",
        "cameraDeviceNames",
        "busy",
        "exposureValueAvailable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Signal 'cameraDeviceListChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'availableChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'cameraDeviceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'cameraFormatChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'busyChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewFinderFrame'
        QtMocHelpers::SignalData<void(const QImage &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'stillCaptureFrames'
        QtMocHelpers::SignalData<void(const QQueue<QImage> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'afWindowChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'settingsChanged'
        QtMocHelpers::SignalData<void(const Settings &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'fpsChanged'
        QtMocHelpers::SignalData<void(float)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 25 },
        }}),
        // Slot 'setCameraDevice'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'setCameraFormat'
        QtMocHelpers::SlotData<void(const QCameraFormat &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'setAfWindow'
        QtMocHelpers::SlotData<void(const QSize &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 32 },
        }}),
        // Slot 'resetAfWindow'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setIso'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'setExposureTime'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 37 },
        }}),
        // Slot 'setExposureValue'
        QtMocHelpers::SlotData<void(float)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 39 },
        }}),
        // Slot 'resetIso'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetExposureTime'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetExposureValue'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setWbTemp'
        QtMocHelpers::SlotData<void(int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Slot 'resetAwb'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setContrast'
        QtMocHelpers::SlotData<void(float)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 47 },
        }}),
        // Slot 'setSaturation'
        QtMocHelpers::SlotData<void(float)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 49 },
        }}),
        // Slot 'resetContrast'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetSaturation'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFps'
        QtMocHelpers::SlotData<void(float)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 25 },
        }}),
        // Slot 'setError'
        QtMocHelpers::SlotData<void(const QString &)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'unsetError'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleCameraAdded'
        QtMocHelpers::SlotData<void(std::shared_ptr<libcamera::Camera>)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 },
        }}),
        // Slot 'handleCameraRemoved'
        QtMocHelpers::SlotData<void(std::shared_ptr<libcamera::Camera>)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 },
        }}),
        // Method 'switchToNextCamera'
        QtMocHelpers::MethodData<bool()>(59, 2, QMC::AccessPublic, QMetaType::Bool),
        // Method 'startCamera'
        QtMocHelpers::MethodData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'stopCamera'
        QtMocHelpers::MethodData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'error'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'cameraDeviceIds'
        QtMocHelpers::PropertyData<QList<QString>>(63, QMetaType::QStringList, QMC::DefaultPropertyFlags, 2),
        // property 'cameraDeviceNames'
        QtMocHelpers::PropertyData<QList<QString>>(64, QMetaType::QStringList, QMC::DefaultPropertyFlags, 2),
        // property 'available'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'busy'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'cameraDevice'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'cameraFormat'
        QtMocHelpers::PropertyData<QCameraFormat>(30, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
        // property 'exposureValueAvailable'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags, 10),
        // property 'exposureValue'
        QtMocHelpers::PropertyData<float>(39, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlasmaCamera, qt_meta_tag_ZN12PlasmaCameraE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlasmaCamera::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlasmaCameraE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlasmaCameraE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12PlasmaCameraE_t>.metaTypes,
    nullptr
} };

void PlasmaCamera::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlasmaCamera *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorChanged(); break;
        case 1: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->cameraDeviceListChanged(); break;
        case 3: _t->availableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->cameraDeviceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->cameraFormatChanged(); break;
        case 6: _t->busyChanged(); break;
        case 7: _t->viewFinderFrame((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 8: _t->stillCaptureFrames((*reinterpret_cast<std::add_pointer_t<QQueue<QImage>>>(_a[1]))); break;
        case 9: _t->afWindowChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 10: _t->settingsChanged((*reinterpret_cast<std::add_pointer_t<Settings>>(_a[1]))); break;
        case 11: _t->fpsChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 12: _t->setCameraDevice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->setCameraFormat((*reinterpret_cast<std::add_pointer_t<QCameraFormat>>(_a[1]))); break;
        case 14: _t->setAfWindow((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 15: _t->resetAfWindow(); break;
        case 16: _t->setIso((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->setExposureTime((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->setExposureValue((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 19: _t->resetIso(); break;
        case 20: _t->resetExposureTime(); break;
        case 21: _t->resetExposureValue(); break;
        case 22: _t->setWbTemp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->resetAwb(); break;
        case 24: _t->setContrast((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 25: _t->setSaturation((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 26: _t->resetContrast(); break;
        case 27: _t->resetSaturation(); break;
        case 28: _t->setFps((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 29: _t->setError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->unsetError(); break;
        case 31: _t->handleCameraAdded((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<libcamera::Camera>>>(_a[1]))); break;
        case 32: _t->handleCameraRemoved((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<libcamera::Camera>>>(_a[1]))); break;
        case 33: { bool _r = _t->switchToNextCamera();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->startCamera(); break;
        case 35: _t->stopCamera(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)()>(_a, &PlasmaCamera::errorChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const QString & )>(_a, &PlasmaCamera::errorOccurred, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)()>(_a, &PlasmaCamera::cameraDeviceListChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(bool )>(_a, &PlasmaCamera::availableChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const QString & )>(_a, &PlasmaCamera::cameraDeviceChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)()>(_a, &PlasmaCamera::cameraFormatChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)()>(_a, &PlasmaCamera::busyChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const QImage & )>(_a, &PlasmaCamera::viewFinderFrame, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const QQueue<QImage> & )>(_a, &PlasmaCamera::stillCaptureFrames, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const QSize & )>(_a, &PlasmaCamera::afWindowChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(const Settings & )>(_a, &PlasmaCamera::settingsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlasmaCamera::*)(float )>(_a, &PlasmaCamera::fpsChanged, 11))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->error(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 2: *reinterpret_cast<QList<QString>*>(_v) = _t->cameraDeviceIds(); break;
        case 3: *reinterpret_cast<QList<QString>*>(_v) = _t->cameraDeviceNames(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isAvailable(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->busy(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->cameraDevice(); break;
        case 7: *reinterpret_cast<QCameraFormat*>(_v) = _t->cameraFormat(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->exposureValueAvailable(); break;
        case 9: *reinterpret_cast<float*>(_v) = _t->exposureValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 6: _t->setCameraDevice(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setCameraFormat(*reinterpret_cast<QCameraFormat*>(_v)); break;
        case 9: _t->setExposureValue(*reinterpret_cast<float*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 9: _t->resetExposureValue(); break;
        default: break;
        }
    }
}

const QMetaObject *PlasmaCamera::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlasmaCamera::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PlasmaCameraE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PlasmaCamera::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 36;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void PlasmaCamera::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PlasmaCamera::errorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PlasmaCamera::cameraDeviceListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PlasmaCamera::availableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PlasmaCamera::cameraDeviceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PlasmaCamera::cameraFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void PlasmaCamera::busyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PlasmaCamera::viewFinderFrame(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void PlasmaCamera::stillCaptureFrames(const QQueue<QImage> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void PlasmaCamera::afWindowChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void PlasmaCamera::settingsChanged(const Settings & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void PlasmaCamera::fpsChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
