// This file is autogenerated. Changes will be overwritten.
#include "EJRQKI7XPS/moc_camerasettings.cpp"
#include "EWIEGA46WW/moc_plasmacamera.cpp"
#include "4XFM65ILQA/moc_worker.cpp"
#include "EWIEGA46WW/moc_plasmacameramanager.cpp"
