/****************************************************************************
** Meta object code from reading C++ file 'worker.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plasma-camera-26.04.0/src/plasmacamera/worker.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QQueue>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'worker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6WorkerE_t {};
} // unnamed namespace

template <> constexpr inline auto Worker::qt_create_metaobjectdata<qt_meta_tag_ZN6WorkerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Worker",
        "ready",
        "",
        "running",
        "finished",
        "errorChanged",
        "errorOccurred",
        "errorString",
        "cameraFormatChanged",
        "supportedFeaturesChanged",
        "viewFinderFrame",
        "QImage",
        "frame",
        "stillCaptureFrames",
        "QQueue<QImage>",
        "frames",
        "init",
        "shutdown",
        "setCamera",
        "std::shared_ptr<libcamera::Camera>",
        "camera",
        "unsetCamera",
        "capture",
        "setSettings",
        "Settings",
        "settings",
        "setError",
        "libcameraError",
        "unsetError",
        "requestNextFrame",
        "processRequestDataAndEmit"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'running'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'cameraFormatChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportedFeaturesChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewFinderFrame'
        QtMocHelpers::SignalData<void(const QImage &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'stillCaptureFrames'
        QtMocHelpers::SignalData<void(const QQueue<QImage> &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shutdown'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setCamera'
        QtMocHelpers::SlotData<void(std::shared_ptr<libcamera::Camera>)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'unsetCamera'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'capture'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setSettings'
        QtMocHelpers::SlotData<void(const Settings &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'setError'
        QtMocHelpers::SlotData<void(int)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'setError'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'unsetError'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestNextFrame'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'processRequestDataAndEmit'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Worker, qt_meta_tag_ZN6WorkerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Worker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WorkerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WorkerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6WorkerE_t>.metaTypes,
    nullptr
} };

void Worker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Worker *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->running(); break;
        case 2: _t->finished(); break;
        case 3: _t->errorChanged(); break;
        case 4: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->cameraFormatChanged(); break;
        case 6: _t->supportedFeaturesChanged(); break;
        case 7: _t->viewFinderFrame((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 8: _t->stillCaptureFrames((*reinterpret_cast<std::add_pointer_t<QQueue<QImage>>>(_a[1]))); break;
        case 9: _t->init(); break;
        case 10: _t->shutdown(); break;
        case 11: _t->setCamera((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<libcamera::Camera>>>(_a[1]))); break;
        case 12: _t->unsetCamera(); break;
        case 13: _t->capture(); break;
        case 14: _t->setSettings((*reinterpret_cast<std::add_pointer_t<Settings>>(_a[1]))); break;
        case 15: _t->setError((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->setError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->unsetError(); break;
        case 18: _t->requestNextFrame(); break;
        case 19: _t->processRequestDataAndEmit(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::running, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::finished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::errorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)(const QString & )>(_a, &Worker::errorOccurred, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::cameraFormatChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)()>(_a, &Worker::supportedFeaturesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)(const QImage & )>(_a, &Worker::viewFinderFrame, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Worker::*)(const QQueue<QImage> & )>(_a, &Worker::stillCaptureFrames, 8))
            return;
    }
}

const QMetaObject *Worker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Worker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6WorkerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Worker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void Worker::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Worker::running()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Worker::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Worker::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Worker::errorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Worker::cameraFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Worker::supportedFeaturesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Worker::viewFinderFrame(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Worker::stillCaptureFrames(const QQueue<QImage> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
